from datetime import datetime as __datetime
from typing import List as __List
from typing import Union as __Union, TYPE_CHECKING
from requests.models import Response as __Response
from requests import Session
import logging as __logging

if TYPE_CHECKING:
    from .enums import Station

from .models import Arrival as __Arrival
from .models import Departure as __Departure
from .models import Trip as __Trip
from .trip import get_trips
from .departures import get_departures
from .arrivals import get_arrivals


__logger = __logging.getLogger("vvspy")


def departures_now(
    station_id: __Union[str, int, "Station"],
    limit: int = 100,
    return_resp: bool = False,
    session: Session = None,
    **kwargs,
) -> __Union[__List[__Departure], __Response, None]:
    """
    Same as `get_departures`
    But `datetime.datetime.now()` is already used as parameter.

    Returns: List[:class:`vvspy.models.Departure`]
    Returns none on webrequest errors or no results found.

    """
    return get_departures(
        station_id=station_id,
        check_time=__datetime.now(),
        limit=limit,
        return_resp=return_resp,
        session=session,
        **kwargs,
    )


def get_departure(
    station_id: __Union[str, int, "Station"],
    check_time: __datetime = None,
    debug: bool = False,
    request_params: dict = None,
    return_resp: bool = False,
    session: Session = None,
    **kwargs,
) -> __Union[__Departure, __Response, None]:
    """
    Same as `get_departures`
    But limited to one obj as result.

    Returns: :class:`vvspy.models.Departure`
    Returns none on webrequest errors or no results found.

    """
    try:
        if return_resp:
            return get_departures(
                station_id=station_id,
                check_time=check_time,
                limit=1,
                debug=debug,
                request_params=request_params,
                return_resp=return_resp,
                session=session,
                **kwargs,
            )
        else:
            return get_departures(
                station_id=station_id,
                check_time=check_time,
                limit=1,
                debug=debug,
                request_params=request_params,
                return_resp=return_resp,
                session=session,
                **kwargs,
            )[0]
    except IndexError as e:  # no results returned
        __logger.error(f"No departures found. {e}")
        raise e
    except TypeError as e:  # none returned | most likely an error
        __logger.error(f"Error on webrequest. {e}")
        raise e


def get_arrival(
    station_id: __Union[str, int, "Station"],
    check_time: __datetime = None,
    debug: bool = False,
    request_params: dict = None,
    return_resp: bool = False,
    session: Session = None,
    **kwargs,
) -> __Union[__Arrival, __Response, None]:
    """
    Same as `get_arrivals`
    But limited to one obj as result.

    Returns: :class:`vvspy.models.Arrival`
    Returns none on webrequest errors or no results found.

    """
    try:
        if return_resp:
            return get_arrivals(
                station_id=station_id,
                check_time=check_time,
                limit=1,
                debug=debug,
                request_params=request_params,
                return_resp=return_resp,
                session=session,
                **kwargs,
            )
        else:
            return get_arrivals(
                station_id=station_id,
                check_time=check_time,
                limit=1,
                debug=debug,
                request_params=request_params,
                return_resp=return_resp,
                session=session,
                **kwargs,
            )[0]
    except IndexError as e:  # no results returned
        __logger.error(f"No arrivals found. {e}")
        raise e
    except TypeError as e:  # none returned | most likely an error
        __logger.error(f"Error on webrequest. {e}")
        raise e


def get_trip(
    origin_station_id: __Union[str, int, "Station"],
    destination_station_id: __Union[str, int, "Station"],
    check_time: __datetime = None,
    debug: bool = False,
    request_params: dict = None,
    return_resp: bool = False,
    session: Session = None,
    **kwargs,
) -> __Union[__Trip, __Response, None]:
    """
    Same as `get_trips`
    But limited to one obj as result.

    Returns: :class:`vvspy.models.Trip`
    Returns none on webrequest errors or no results found.

    """
    try:
        if return_resp:
            return get_trips(
                origin_station_id=origin_station_id,
                destination_station_id=destination_station_id,
                check_time=check_time,
                limit=1,
                debug=debug,
                request_params=request_params,
                return_resp=return_resp,
                session=session,
                **kwargs,
            )
        else:
            return get_trips(
                origin_station_id=origin_station_id,
                destination_station_id=destination_station_id,
                check_time=check_time,
                limit=1,
                debug=debug,
                request_params=request_params,
                session=session,
                **kwargs,
            )[0]
    except IndexError as e:
        __logger.error(f"No trips found. {e}")
        raise e
    except TypeError as e:
        __logger.error(f"Error on webrequest. {e}")
        raise e



from datetime import datetime

from .serving_line import ServingLine
from .line_operator import LineOperator


class Departure:
    r"""

     Departure object from a departure request of one station.

    Attributes
    -----------

    raw :class:`dict`
        Raw dict received by the API.
    stop_id :class:`str`
        Station_id of the departure.
    cancelled :class:`bool`
        If the departure is cancelled.
    realtime_status :class:`str`
        Realtime status of the departure.
    x :class:`str`
        Coordinates of the station.
    y :class:`str`
        Coordinates of the station.
    map_name :class:`str`
        Map name the API works on.
    area :class:`str`
        The area of the station (unsure atm)
    platform :class:`str`
        Platform / track of the departure.
    platform_name :class:`str`
        name of the platform.
    stop_name :class:`str`
        name of the station.
    name_wo :class:`str`
        name of the station.
    countdown :class:`int`
        minutes until departure.
    datetime :class:`datetime.datetime`
        Planned departure datetime.
    real_datetime :class:`datetime.datetime`
        Estimated departure datetime (equal to ``self.datetime`` if no realtime data is available).
    delay :class:`int`
        Delay of departure in minutes.
    serving_line :class:`ServingLine`
        line of the incoming departure.
    operator :class:`LineOperator`
        Operator of the incoming departure.
    stop_infos: Optional[:class:`dict`]
        All related info to the station (e.g. maintenance work).
    line_infos Optional[:class:`dict`]
        All related info to the station (e.g. maintenance work).
    """

    def __init__(self, **kwargs):
        self.stop_id = kwargs.get("stopID")
        self.x = kwargs.get("x")
        self.y = kwargs.get("y")
        self.realtime_status = kwargs.get("realtimeStatus")
        self.cancelled = self.realtime_status == "DEPARTURE_CANCELLED"
        self.map_name = kwargs.get("mapName")
        self.area = kwargs.get("area")
        self.platform = kwargs.get("platform")
        self.platform_name = kwargs.get("platformName")
        self.stop_name = kwargs.get("stopName")
        self.name_wo = kwargs.get("nameWO")
        self.point_type = kwargs.get("pointType")
        self.countdown = int(kwargs.get("countdown", "0"))
        dt = kwargs.get("dateTime")
        if dt:
            try:
                self.datetime = datetime(
                    year=int(dt.get("year", datetime.now().year)),
                    month=int(dt.get("month", datetime.now().month)),
                    day=int(dt.get("day", datetime.now().day)),
                    hour=int(dt.get("hour", datetime.now().hour)),
                    minute=int(dt.get("minute", datetime.now().minute)),
                )
            except ValueError:
                pass
        else:
            self.datetime = None
        r_dt = kwargs.get("realDateTime")
        if r_dt:
            try:
                self.real_datetime = datetime(
                    year=int(r_dt.get("year", datetime.now().year)),
                    month=int(r_dt.get("month", datetime.now().month)),
                    day=int(r_dt.get("day", datetime.now().day)),
                    hour=int(r_dt.get("hour", datetime.now().hour)),
                    minute=int(r_dt.get("minute", datetime.now().minute)),
                )
            except ValueError:
                pass
        else:
            self.real_datetime = self.datetime

        self.delay = int((self.real_datetime - self.datetime).total_seconds() / 60)
        self.serving_line = ServingLine(**kwargs.get("servingLine", {}))
        self.operator = LineOperator(**kwargs.get("operator", {}))

        # inserted raw
        self.raw = kwargs
        self.stop_infos = kwargs.get("stopInfos")
        self.line_infos = kwargs.get("lineInfos")

    def __str__(self):
        pre = ""
        if self.delay:
            pre = "[Delayed] "
        if self.cancelled:
            pre = "[Cancelled] "
        if self.real_datetime.date() == datetime.now().date():
            return f"{pre}[{str(self.real_datetime.strftime('%H:%M'))}] {self.serving_line}"
        return f"{pre}[{str(self.real_datetime)}] {self.serving_line}"



class ServingLine:
    r"""

        Describes the line, departing or arriving in a Departure/Arrival result.

        Attributes
        -----------

        raw :class:`dict`
            Raw dict received by the API.
        key :class:`str`
            key (most likely an ID) of the line.
        code :class:`str`
            code (most likely type) of the line.
        number :class:`str`
            number of line (e.g. U12).
        symbol :class:`str`
            symbol displayed on the transport itself (e.g. U12).
        mot_type :class:`str`
            ~
        mt_sub_code :class:`str`
            ~
        real_time :class:`bool`
            whether or not the transport supports realtime tracking.
        direction :class:`str`
            Last station the transport is heading to.
        direction :class:`str`
            Last station the transport is heading to.
        direction_from :class:`str`
            Starting station of the transport.
        name :class:`str`
            name of the line type (e.g. Stadtbahn).
        train_num :class:`str`
            Last station the transport is heading to.
        delay Optional[:class:`str`]
            Minutes of delay.
        li_erg_ri_proj :class:`dict`
            Detailed line information (e.g. network)
        dest_id :class:`str`
            station id of the destination
        stateless :class:`str`
            ~
    """
    def __init__(self, **kwargs):
        self.raw = kwargs
        self.key = kwargs.get("key")
        self.code = kwargs.get("code")
        self.number = kwargs.get("number")
        self.symbol = kwargs.get("symbol")
        self.mot_type = kwargs.get("motType")
        self.mt_sub_code = kwargs.get("mtSubCode")
        try:
            self.real_time = bool(int(kwargs.get("realtime", "0")))
        except ValueError:
            self.real_time = False
        self.direction = kwargs.get("direction")
        self.direction_from = kwargs.get("directionFrom")
        self.name = kwargs.get("trainName", kwargs.get("name"))
        self.delay = kwargs.get("delay")
        self.li_erg_ri_proj = kwargs.get("liErgRiProj")
        self.dest_id = kwargs.get("destID")
        self.stateless = kwargs.get("stateless")

    def __str__(self):
        return f"[{self.symbol}]: {self.direction_from} - {self.direction}"



class LineOperator:
    r"""

       Describes the operator of a :class:`Connection`.

       Attributes
       -----------

       raw :class:`dict`
           Raw dict received by the API.
       id :class:`str`
           id of the operator.
       name :class:`str`
           display name of the operator.
       public_code :class:`str`
           public_code of the operator.
    """
    def __init__(self, **kwargs):
        self.raw = kwargs
        self.id = kwargs.get("code", kwargs.get("id"))
        self.name = kwargs.get("name")
        self.public_code = kwargs.get("publicCode")

    def __str__(self):
        return f"{self.name} ({self.id})"



from datetime import datetime


class Destination:
    r"""

        Describes the destination of a :class:`Connection`.

        Attributes
        -----------

        raw :class:`dict`
            Raw dict received by the API.
        is_global_id :class:`bool`
            ~
        id :class:`str`
            station id of the destination station
        name :class:`str`
            name of the destination station
        disassembled_name Optional[:class:`str`]
            detailed name of the destination station.
        type :class:`str`
            type of the destination station. (e.g. bus, track)
        point_type Optional[:class:`str`]
            ~
        coord List[:class:`int`]
            coords of the station
        niveau :class:`int`
            ~
        parent :class:`dict`
            ~
        arrival_time_planned :class:`datetime.datetime`
            Time planned of arrival.
        arrival_time_estimated :class:`datetime.datetime`
            Time estimated with realtime info (same as `arrival_time_planned` if no realtime data is available).
        delay :class:`int`
            Minutes of delay.
        properties :class:`dict`
            misc info about the destination.
    """
    def __init__(self, **kwargs):
        self.is_global_id = kwargs.get("isGlobalId")
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.disassembled_name = kwargs.get("disassembledName"),
        self.type = kwargs.get("type")
        self.point_type = kwargs.get("pointType")
        self.coord = tuple(kwargs.get("coord", []))
        self.niveau = kwargs.get("niveau")
        self.parent = kwargs.get("parent")
        self.arrival_time_planned = datetime.strptime(kwargs.get("arrivalTimePlanned", "")[:-1], "%Y-%m-%dT%H:%M:%S")
        self.arrival_time_estimated = datetime.strptime(kwargs.get("arrivalTimeEstimated", "")[:-1],
                                                        "%Y-%m-%dT%H:%M:%S")
        delta = self.arrival_time_estimated - self.arrival_time_planned
        self.delay = int(delta.total_seconds() / 60)

        # inserted raw
        self.raw = kwargs
        self.properties = kwargs.get("properties")



from datetime import datetime

from .origin import Origin
from .destination import Destination
from .transportation import Transportation


class Connection:
    r"""

        Several connections describe one :class:`Trip`.


        Attributes
        -----------

        raw :class:`dict`
            Raw dict received by the API.
        duration :class:`int`
            seconds this connection takes
        is_realtime_controlled :class:`bool`
            whether or not this connection has realtime tracking
        origin :class:`Origin`
            Origin, where this connection starts
        destination :class:`Destination`
            Where this connection is heading to
        transportation :class:`Transportation`
            Transportation info of this connection
        stop_sequence Optional[List[:class:`dict`]]
            stop sequence of this connection
        foot_path_info Optional[]
            Info if you really want to walk ?
        infos Optional[List[]]
            ~
        coords Optional[List[List[:class:`int`]]]
            coords of this connection
        path_description Optional[]
            ~
        interchange Optional[]
            ~
        properties Optional[:class:`dict`]
            misc info about this connection
    """

    def __init__(self, **kwargs):
        self.duration = kwargs.get("duration")
        self.is_realtime_controlled = kwargs.get("isRealtimeControlled", False)
        self.origin = Origin(**kwargs.get("origin"))
        self.destination = Destination(**kwargs.get("destination"))
        self.transportation = Transportation(**kwargs.get("transportation"))

        # inserted raw
        self.raw = kwargs
        self.stop_sequence = kwargs.get("stopSequence")
        self.foot_path_info = kwargs.get("footPathInfo")
        self.infos = kwargs.get("infos")
        self.coords = kwargs.get("coords")
        self.path_description = kwargs.get("pathDescription")
        self.interchange = kwargs.get("interchange")
        self.properties = kwargs.get("properties")

    def __str__(self):
        dep_pre = "[Delayed] " if self.origin.delay else ""
        arr_pre = "[Delayed] " if self.destination.delay else ""
        if self.origin.departure_time_estimated.date() == datetime.now().date():
            return f"[{self.transportation.disassembled_name}]: " \
                f"{dep_pre}[{self.origin.departure_time_estimated.strftime('%H:%M')}] @ {self.origin.name} - " \
                f"{arr_pre}[{self.destination.arrival_time_estimated.strftime('%H:%M')}] @ {self.destination.name}"
        return f"[{self.transportation.disassembled_name}]: " \
            f"{dep_pre}[{self.origin.departure_time_estimated}] @ {self.origin.name} - " \
            f"{arr_pre}[{self.destination.arrival_time_estimated}] @ {self.destination.name}"



from datetime import datetime
from requests.models import Response
from typing import Union, List, TYPE_CHECKING
from enum import Enum
import requests
import json
import logging as __logging

if TYPE_CHECKING:
    from .enums import Station
from .models import Trip

__API_URL = "https://www3.vvs.de/mngvvs/XML_TRIP_REQUEST2"
__logger = __logging.getLogger("vvspy")


def get_trips(
    origin_station_id: Union[str, int, "Station"],
    destination_station_id: Union[str, int, "Station"],
    check_time: datetime = None,
    limit: int = 100,
    request_params: dict = None,
    return_response: bool = False,
    session: requests.Session = None,
    **kwargs,
) -> Union[List[Trip], Response, None]:
    r"""

    Returns: List[:class:`vvspy.models.Trip`]
    Returns none on webrequest errors.

    Examples
    --------
    Basic usage:

    .. code-block:: python

        results = vvspy.get_trips("5006115", "5006465", limit=3)  # Stuttgart main station to Zuffenhausen

    Set proxy for request:

    .. code-block:: python

        proxies = {}  # see https://stackoverflow.com/a/8287752/9850709
        results = vvspy.get_arrivals("5006115", "5006465", request_params={"proxies": proxies})

    Parameters
    ----------
        station_id Union[:class:`int`, :class:`str`, :class:`vvspy.enums.Station`]
            Station you want to get trips from.
        check_time Optional[:class:`datetime.datetime`]
            Time you want to check.
            default datetime.now()
        limit Optional[:class:`int`]
            Limit request/result on this integer.
            default 100
        request_params Optional[:class:`dict`]
            params parsed to the api request (e.g. proxies)
            default {}
        return_response Optional[:class:`bool`]
            if set, the function returns the response object of the API request.
        session Optional[:class:`requests.Session`]
            if set, uses a given requests.session object for requests
        kwargs Optional[:class:`dict`]
            Check trips.py to see all available kwargs.
    """

    if not check_time:
        check_time = datetime.now()
    if request_params is None:
        request_params = dict()

    params = {
        "SpEncId": kwargs.get("SpEncId", "0"),
        "calcOneDirection": kwargs.get("calcOneDirection", "1"),
        "changeSpeed": kwargs.get("changeSpeed", "normal"),
        "computationType": kwargs.get("computationType", "sequence"),
        "coordOutputFormat": kwargs.get("coordOutputFormat", "EPSG:4326"),
        "cycleSpeed": kwargs.get("cycleSpeed", "14"),
        "deleteAssignedStops": kwargs.get("deleteAssignedStops", "0"),
        "deleteITPTWalk": kwargs.get("deleteITPTWalk", "0"),
        "descWithElev": kwargs.get("descWithElev", "1"),
        "illumTransfer": kwargs.get("illumTransfer", "on"),
        "imparedOptionsActive": kwargs.get("imparedOptionsActive", "1"),
        "itOptionsActive": kwargs.get("itOptionsActive", "1"),
        "itdDate": check_time.strftime("%Y%m%d"),
        "itdTime": check_time.strftime("%H%M"),
        "language": kwargs.get("language", "de"),
        "locationServerActive": kwargs.get("locationServerActive", "1"),
        "macroWebTrip": kwargs.get("macroWebTrip", "true"),
        "name_destination": (
            destination_station_id.value
            if isinstance(destination_station_id, Enum)
            else str(destination_station_id)
        ),
        "name_origin": (
            origin_station_id.value
            if isinstance(origin_station_id, Enum)
            else str(origin_station_id)
        ),
        "noElevationProfile": kwargs.get("noElevationProfile", "1"),
        "noElevationSummary": kwargs.get("noElevationSummary", "1"),
        "outputFormat": "rapidJSON",
        "outputOptionsActive": "1",
        "ptOptionsActive": kwargs.get("ptOptionsActive", "1"),
        "routeType": kwargs.get("routeType", "leasttime"),
        "searchLimitMinutes": kwargs.get("searchLimitMinutes", "360"),
        "securityOptionsActive": kwargs.get("securityOptionsActive", "1"),
        "serverInfo": kwargs.get("serverInfo", "1"),
        "showInterchanges": kwargs.get("showInterchanges", "1"),
        "trITArrMOT": kwargs.get("trITArrMOT", "100"),
        "trITArrMOTvalue": kwargs.get("trITArrMOTvalue", "15"),
        "trITDepMOT": kwargs.get("trITDepMOT", "100"),
        "trITDepMOTvalue": kwargs.get("trITDepMOTvalue", "15"),
        "tryToFindLocalityStops": kwargs.get("tryToFindLocalityStops", "1"),
        "type_destination": kwargs.get("type_destination", "any"),
        "type_origin": kwargs.get("type_origin", "any"),
        "useElevationData": kwargs.get("useElevationData", "1"),
        "useLocalityMainStop": kwargs.get("useLocalityMainStop", "0"),
        "useRealtime": kwargs.get("useRealtime", "1"),
        "useUT": kwargs.get("useUT", "1"),
        "version": kwargs.get("version", "10.2.10.139"),
        "w_objPrefAl": kwargs.get("w_objPrefAl", "12"),
        "w_regPrefAm": kwargs.get("w_regPrefAm", "1"),
    }

    if session:
        r = session.get(__API_URL, **{**request_params, **{"params": params}})
    else:
        r = requests.get(__API_URL, **{**request_params, **{"params": params}})

    __logger.debug(
        f"Request took {r.elapsed.total_seconds()}s and returned {r.status_code}"
    )

    if r.status_code != 200:
        __logger.error("Error in API request")
        __logger.error(f"Request: {r.status_code}")
        __logger.error(f"{r.text}")
        raise Exception(f"Error in API request: {r.status_code}")

    if return_response:
        return r

    __logger.debug("Initializing parsing of response...")

    try:
        r.encoding = "UTF-8"
        return _parse_response(r.json(), limit)
    except json.decoder.JSONDecodeError as e:
        __logger.error(
            "Error in API request. Received invalid JSON. Status code: %s",
            r.status_code,
        )
        raise e


def _parse_response(result: dict, limit: int = 100) -> Union[List[Trip], None]:
    parsed_trips = []
    if not result or "journeys" not in result or not result["journeys"]:
        return []  # no trips found
    for trip in result["journeys"][: int(limit)]:
        parsed_trips.append(Trip(**trip))

    return parsed_trips



from .connection import Connection


class Trip:
    r"""

        Result object from a trip request from one station to another including interchanges

        Attributes
        -----------

        raw :class:`dict`
            Raw dict received by the API.
        connections List[:class:`Connection`]
            List of connections the trip consists of.
        duration :class:`int`
            seconds the trip takes overall.
        zones Optional[List[:class:`str`]]
            List of zones this trip goes through.
        fare Optional[:class:`dict`]
            misc info about this trip, ticket prices, etc.
    """

    def __init__(self, **kwargs):
        self.connections = []
        for connection in kwargs.get("legs", []):
            self.connections.append(Connection(**connection))

        self.duration = sum([x.duration for x in self.connections if x.duration])

        try:
            self.zones = kwargs.get("fare", {}).get("zones", [])[0].get("zones", [])
        except IndexError:
            self.zones = []
        # inserted raw
        self.raw = kwargs
        self.fare = kwargs.get("fare")

    def __str__(self):
        return f"Connection ({int(self.duration / 60)} minutes):\n" \
               + '\n'.join([str(x) for x in self.connections])



from typing import List, Union, TYPE_CHECKING
from datetime import datetime
from requests.models import Response
from enum import Enum
import requests
import json
import logging as __logging

if TYPE_CHECKING:
    from .enums import Station
from .models import Arrival

_API_URL = "http://www3.vvs.de/vvs/widget/XML_DM_REQUEST?"
__logger = __logging.getLogger("vvspy")


def get_arrivals(
    station_id: Union[str, int, "Station"],
    check_time: datetime = None,
    limit: int = 100,
    request_params: dict = None,
    return_response: bool = False,
    session: requests.Session = None,
    **kwargs,
) -> Union[List[Arrival], Response, None]:
    r"""

    Returns: List[:class:`vvspy.models.Arrival`]
    Returns none on webrequest errors.

    Examples
    --------
    Basic usage:

    .. code-block:: python

        results = vvspy.get_arrivals("5006115", limit=3)  # Stuttgart main station

    Set proxy for request:

    .. code-block:: python

        proxies = {}  # see https://stackoverflow.com/a/8287752/9850709
        results = vvspy.get_arrivals("5006115", request_params={"proxies": proxies})

    Parameters
    ----------
        station_id Union[:class:`int`, :class:`str`, :class:`vvspy.enums.Station`]
            Station you want to get arrivals from.
        check_time Optional[:class:`datetime.datetime`]
            Time you want to check.
            default datetime.now()
        limit Optional[:class:`int`]
            Limit request/result on this integer.
            default 100
        request_params Optional[:class:`dict`]
            params parsed to the api request (e.g. proxies)
            default {}
        return_response Optional[:class:`bool`]
            if set, the function returns the response object of the API request.
        session Optional[:class:`requests.Session`]
            if set, uses a given requests.session object for requests
        kwargs Optional[:class:`dict`]
            Check arrivals.py to see all available kwargs.
    """

    if not check_time:
        check_time = datetime.now()
    if request_params is None:
        request_params = dict()
    params = {
        "locationServerActive": kwargs.get(
            "locationServerActive", 1
        ),  # typo from zocationServerActive ?!
        "lsShowTrainsExplicit": kwargs.get("lsShowTrainsExplicit", 1),
        "stateless": kwargs.get("stateless", 1),
        "language": kwargs.get("language", "de"),
        "SpEncId": kwargs.get("SpEncId", 0),
        "anySigWhenPerfectNoOtherMatches": kwargs.get(
            "anySigWhenPerfectNoOtherMatches", 1
        ),
        "limit": limit,
        "depArr": "arrival",
        "type_dm": kwargs.get("type_dm", "any"),
        "anyObjFilter_dm": kwargs.get("anyObjFilter_dm", 2),
        "deleteAssignedStops": kwargs.get("deleteAssignedStops", 1),
        "name_dm": (
            station_id.value if isinstance(station_id, Enum) else str(station_id)
        ),
        "mode": kwargs.get("mode", "direct"),
        "dmLineSelectionAll": kwargs.get("dmLineSelectionAll", 1),
        "useRealtime": kwargs.get("useRealtime", 1),  # live delay
        "outputFormat": kwargs.get("outputFormat", "json"),
        "coordOutputFormat": kwargs.get("coordOutputFormat", "WGS84[DD.ddddd]"),
        "itdDateTimeDepArr": "arr",
        "itdDateYear": check_time.strftime("%Y"),
        "itdDateMonth": check_time.strftime("%m"),
        "itdDateDay": check_time.strftime("%d"),
        "itdTimeHour": check_time.strftime("%H"),
        "itdTimeMinute": check_time.strftime("%M"),
        "itdTripDateTimeDepArr": "arr",
    }

    if session:
        r = session.get(_API_URL, **{**request_params, **{"params": params}})
    else:
        r = requests.get(_API_URL, **{**request_params, **{"params": params}})

    __logger.debug(
        f"Request took {r.elapsed.total_seconds()}s and returned {r.status_code}"
    )

    if r.status_code != 200:
        __logger.error("Error in API request")
        __logger.error(f"Request: {r.status_code}")
        __logger.error(f"{r.text}")
        raise Exception(f"Error in API request: {r.status_code}")

    if return_response:
        return r

    __logger.debug("Initializing parsing of response...")

    try:
        r.encoding = "UTF-8"
        return _parse_response(r.json())
    except json.decoder.JSONDecodeError as e:
        __logger.error(
            "Error in API request. Received invalid JSON. Status code: %s",
            r.status_code,
        )
        raise e


def _parse_response(result: dict) -> List[Arrival]:
    parsed_response = []

    if (
        not result or "arrivalList" not in result or not result["arrivalList"]
    ):  # error in response/request
        return []  # no results

    if isinstance(result["arrivalList"], dict):  # one result
        parsed_response.append(Arrival(**result["arrivalList"]["arrival"]))
    elif isinstance(result["arrivalList"], list):  # multiple result
        for arrival in result["arrivalList"]:
            parsed_response.append(Arrival(**arrival))

    return parsed_response



from .arrival import Arrival
from .departure import Departure
from .arrival import Arrival
from .trip import Trip
from .serving_line import ServingLine
from .line_operator import LineOperator
from .connection import Connection
from .origin import Origin
from .destination import Destination
from .transportation import Transportation



from .line_operator import LineOperator


class Transportation:
    r"""

        Describes info about transportation of a :class:`Connection`.

        Attributes
        -----------

        raw :class:`dict`
            Raw dict received by the API.
        id :class:`str`
            id of the transportation.
        name :class:`str`
            name of the transportation.
        disassembled_name :class:`str`
            detailed name of the transportation.
        number :class:`str`
            line number of the transportation.
        description :class:`str`
            description, most of the time the string that is displayed on the bus/train itself.
        product :class:`dict`
            describes the mean of transport (bus, train, etc.)
        operator :class:`LineOperator`
            describes the Operator of the transport.
        destination :class:`dict`
            destination of the transport.
        properties :class:`dict`
            misc info about the transport.
    """

    def __init__(self, **kwargs):
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.disassembled_name = kwargs.get("disassembledName", "Walk")
        self.number = kwargs.get("number")
        self.description = kwargs.get("description")
        self.product = kwargs.get("product")
        self.operator = LineOperator(**kwargs.get("operator", {}))
        self.destination = kwargs.get("destination")

        # inserted raw
        self.raw = kwargs
        self.properties = kwargs.get("properties")



from setuptools import setup, find_packages

with open("readme.md", "r") as fh:
    long_description = fh.read()

setup(
    name="vvspy",
    version="2.3.1",
    license="MIT",
    description="API Wrapper for VVS (Verkehrsverbund Stuttgart)",
    author="zaanposni",
    author_email="vvspy@zaanposni.com",
    url="https://github.com/zaanposni/vvspy",
    keywords=["vvs", "api", "stuttgart", "wrapper", "json", "rest", "efa", "python"],
    packages=find_packages(exclude=["*tests"]),
    package_data={"vvspy": ["vvspy/*"]},
    python_requires=">=3.8",
    install_requires=[
        "requests",
        "typing",
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Operating System :: OS Independent",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
)
